mod api;
mod root;

use std::sync::Arc;

use axum::{body::Body, Router};
use http::Request;
use sqlx::{postgres::PgConnectOptions, PgPool};
use tower_http::trace::{DefaultOnRequest, DefaultOnResponse, TraceLayer};
use tracing::{field, span, Level};

use crate::config::Config;

pub async fn build_app(config: Config) -> eyre::Result<Router> {
    let db = PgPool::connect_with(
        PgConnectOptions::new()
            .application_name("ncpn")
            .socket(&config.db_socket)
            .username(&config.db_username)
            .database(&config.db_database),
    )
    .await?;

    let config = Arc::new(config);

    Ok(root::router(db.clone(), config.clone())
        .nest("/api", api::router(db, config))
        .layer(
            TraceLayer::new_for_http()
                .make_span_with(|request: &Request<Body>| {
                    span!(
                        Level::INFO,
                        "http-request",
                        uri = field::display(request.uri()),
                    )
                })
                .on_request(DefaultOnRequest::new().level(Level::DEBUG))
                .on_response(DefaultOnResponse::new().level(Level::INFO)),
        ))
}
